@@ -16,6 +16,8 @@ use Composer\IO\IOInterface;
 use Composer\Json\JsonFile;
 use Composer\Package\Loader\ArrayLoader;
 use Composer\Package\Loader\LoaderInterface;
+use Composer\Util\Tar;
+use Composer\Util\Zip;
 
 /**
  * @author Serge Smertin <serg.smertin@gmail.com>
@@ -42,6 +44,11 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
         $this->repoConfig = $repoConfig;
     }
 
+    public function getRepoName()
+    {
+        return 'artifact repo ('.$this->lookup.')';
+    }
+
     public function getRepoConfig()
     {
         return $this->repoConfig;
@@ -60,7 +67,7 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
 
         $directory = new \RecursiveDirectoryIterator($path, \RecursiveDirectoryIterator::FOLLOW_SYMLINKS);
         $iterator = new \RecursiveIteratorIterator($directory);
-        $regex = new \RegexIterator($iterator, '/^.+\.(zip|phar)$/i');
+        $regex = new \RegexIterator($iterator, '/^.+\.(zip|phar|tar|gz|tgz)$/i');
         foreach ($regex as $file) {
             /* @var $file \SplFileInfo */
             if (!$file->isFile()) {
@@ -80,78 +87,36 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
         }
     }
 
-    /**
-     * Find a file by name, returning the one that has the shortest path.
-     *
-     * @param \ZipArchive $zip
-     * @param string $filename
-     * @return bool|int
-     */
-    private function locateFile(\ZipArchive $zip, $filename)
-    {
-        $indexOfShortestMatch = false;
-        $lengthOfShortestMatch = -1;
-
-        for ($i = 0; $i < $zip->numFiles; $i++) {
-            $stat = $zip->statIndex($i);
-            if (strcmp(basename($stat['name']), $filename) === 0) {
-                $directoryName = dirname($stat['name']);
-                if ($directoryName == '.') {
-                    //if composer.json is in root directory
-                    //it has to be the one to use.
-                    return $i;
-                }
-
-                if (strpos($directoryName, '\\') !== false ||
-                   strpos($directoryName, '/') !== false) {
-                    //composer.json files below first directory are rejected
-                    continue;
-                }
-
-                $length = strlen($stat['name']);
-                if ($indexOfShortestMatch === false || $length < $lengthOfShortestMatch) {
-                    //Check it's not a directory.
-                    $contents = $zip->getFromIndex($i);
-                    if ($contents !== false) {
-                        $indexOfShortestMatch = $i;
-                        $lengthOfShortestMatch = $length;
-                    }
-                }
-            }
-        }
-
-        return $indexOfShortestMatch;
-    }
-
     private function getComposerInformation(\SplFileInfo $file)
     {
-        $zip = new \ZipArchive();
-        if ($zip->open($file->getPathname()) !== true) {
+        $json = null;
+        $fileType = null;
+        $fileExtension = pathinfo($file->getPathname(), PATHINFO_EXTENSION);
+        if (in_array($fileExtension, array('gz', 'tar', 'tgz'), true)) {
+            $fileType = 'tar';
+        } elseif ($fileExtension === 'zip') {
+            $fileType = 'zip';
+        } else {
+            throw new \RuntimeException('Files with "'.$fileExtension.'" extensions aren\'t supported. Only ZIP and TAR/TAR.GZ/TGZ archives are supported.');
+        }
+
+        try {
+            if ($fileType === 'tar') {
+                $json = Tar::getComposerJson($file->getPathname());
+            } else {
+                $json = Zip::getComposerJson($file->getPathname());
+            }
+        } catch (\Exception $exception) {
+            $this->io->write('Failed loading package '.$file->getPathname().': '.$exception->getMessage(), false, IOInterface::VERBOSE);
+        }
+
+        if (null === $json) {
             return false;
         }
 
-        if (0 == $zip->numFiles) {
-            $zip->close();
-
-            return false;
-        }
-
-        $foundFileIndex = $this->locateFile($zip, 'composer.json');
-        if (false === $foundFileIndex) {
-            $zip->close();
-
-            return false;
-        }
-
-        $configurationFileName = $zip->getNameIndex($foundFileIndex);
-        $zip->close();
-
-        $composerFile = "zip://{$file->getPathname()}#$configurationFileName";
-        $json = file_get_contents($composerFile);
-
-        $package = JsonFile::parseJson($json, $composerFile);
+        $package = JsonFile::parseJson($json, $file->getPathname().'#composer.json');
         $package['dist'] = array(
-            'type' => 'zip',
+            'type' => $fileType,
             'url' => strtr($file->getPathname(), '\\', '/'),
             'shasum' => sha1_file($file->getRealPath()),
         );
