@@ -19,8 +19,6 @@ use Composer\CaBundle\CaBundle;
  */
 final class TlsHelper
 {
-    private static $useOpensslParse;
-
     /**
      * Match hostname against a certificate.
      *
@@ -100,7 +98,7 @@ final class TlsHelper
      * By Kevin McArthur of StormTide Digital Studios Inc.
      * @KevinSMcArthur / https://github.com/StormTide
      *
-     * See http://tools.ietf.org/html/draft-ietf-websec-key-pinning-02
+     * See https://tools.ietf.org/html/draft-ietf-websec-key-pinning-02
      *
      * This method was adapted from Sslurp.
      * https://github.com/EvanDotPro/Sslurp
